@@ -30,14 +30,16 @@ from googlecloudsdk.api_lib.storage import request_config_factory
 from googlecloudsdk.api_lib.storage.gcs_grpc import grpc_util
 from googlecloudsdk.api_lib.storage.gcs_grpc import metadata_util
 from googlecloudsdk.api_lib.storage.gcs_grpc import retry_util
+from googlecloudsdk.api_lib.storage.gcs_grpc_bidi_streaming import retry_util as bidi_retry_util
 from googlecloudsdk.command_lib.storage import errors as command_errors
 from googlecloudsdk.command_lib.storage import fast_crc32c_util
 from googlecloudsdk.command_lib.storage import hash_util
+from googlecloudsdk.command_lib.storage import posix_util
 from googlecloudsdk.command_lib.storage.resources import resource_reference
 from googlecloudsdk.command_lib.storage.tasks.cp import copy_util
 from googlecloudsdk.command_lib.util import crc32c
-from googlecloudsdk.core import gapic_util
 from googlecloudsdk.core import log
+from googlecloudsdk.core import properties
 import six
 
 # TODO: b/441010615 - Remove this constant once the flush size is configurable
@@ -66,6 +68,7 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
       ),
       start_offset: int = 0,
       delegator: cloud_api.CloudApi | None = None,
+      posix_to_set: posix_util.PosixAttributes | None = None,
   ):
     """Initializes _Upload.
 
@@ -81,6 +84,7 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
         the data should be written.
       delegator: The client used to make non-bidi streaming or metadata API
         calls.
+      posix_to_set: POSIX attributes to set on the destination object.
     """
     self._client = client
     self._source_stream = source_stream
@@ -96,6 +100,14 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
     )
     self._buffer = collections.deque(maxlen=self._get_max_buffer_size())
     self._initial_request = None
+    self._should_finalize_writes = (
+        properties.VALUES.storage.bidi_streaming_finalize_writes.GetBool()
+    )
+    self._finalized_resource = None
+    self._redirection_handler = bidi_retry_util.BidiRedirectedTokenErrorHandler(
+        self._client, self._destination_resource
+    )
+    self._posix_to_set = posix_to_set
 
   def _get_max_buffer_size(self):
     """Returns the maximum buffer size."""
@@ -149,8 +161,39 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
       ):
         self._start_offset = response.persisted_size
         break
+      elif response.resource:
+        self._finalized_resource = (
+            metadata_util.get_object_resource_from_grpc_object(
+                response.resource
+            )
+        )
+        break
+
     self._buffer.clear()
 
+  def _half_close_bidi_write_rpc(self, bidi_write_rpc) -> None:
+    """Performs a client-side half-close of the bidi write RPC stream.
+
+    This is needed when `_should_finalize_writes` is false to signal the server
+    that the client is done sending requests. It also processes any remaining
+    server responses. The full RPC stream closure(cancel) is handled by
+    `bidi_write_rpc.close()`.
+
+    Args:
+      bidi_write_rpc: The bidi write RPC object.
+    """
+    if self._should_finalize_writes:
+      return
+    # Signal the server to half-close the stream.
+    bidi_write_rpc.requests_done()
+    while True:
+      try:
+        response = bidi_write_rpc.recv()
+      except StopIteration:
+        break
+      if response.persisted_size == self._uploaded_so_far:
+        self._start_offset = response.persisted_size
+
   def _update_initial_request(self, response):
     """Updates the initial request for the bidi write RPC.
 
@@ -188,39 +231,13 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
         ),
     )
 
-  def _get_updated_bidi_write_rpc_if_retrying(self, bidi_write_rpc):
-    """Returns the updated bidi write RPC for resuming an upload."""
-    # If the call is None, it means that the RPC was not initialized, and its a
-    # new upload. In this case we need not initialize the RPC again. As it is
-    # already done by the caller.
-    if bidi_write_rpc.call is not None:
-      # If the call is not None, it means that the RPC was initialized before
-      # and we need to create a new RPC with the updated initial request for
-      # retrying the upload.
-      bidi_write_rpc = gapic_util.MakeBidiRpc(
-          self._client,
-          self._client.storage.bidi_write_object,
-          initial_request=self._initial_request,
-          metadata=metadata_util.get_bucket_name_routing_header(
-              grpc_util.get_full_bucket_name(
-                  self._destination_resource.storage_url.bucket_name
-              )
-          ),
-      )
-    return bidi_write_rpc
-
   @retry_util.grpc_default_retryer
-  def _bidi_write_appendable_object(
-      self, bidi_write_rpc, resuming_upload: bool = False
-  ):
+  def _bidi_write_appendable_object(self, resuming_upload: bool = False):
     """Performs the bidi write RPC for an appendable object."""
-    bidi_write_rpc = self._get_updated_bidi_write_rpc_if_retrying(
-        bidi_write_rpc
+    bidi_write_rpc = self._redirection_handler.start_bidi_rpc_with_retry_on_redirected_token_error(
+        self._initial_request
     )
     try:
-      bidi_write_rpc.open()
-      # TODO: b/440507899 - Add support for retrying an upload with a token.
-      # (Redirected token error handling) and/or error handling/retries.
       response = bidi_write_rpc.recv()
       self._update_initial_request(response)
 
@@ -262,6 +279,7 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
                 ),
                 flush=should_flush,
                 state_lookup=should_flush,
+                finish_write=(finish_write and self._should_finalize_writes),
             )
         )
         self._uploaded_so_far += length_of_data
@@ -269,8 +287,14 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
           self._send_buffer_to_bidi_write_rpc(bidi_write_rpc)
 
         if finish_write:
+          self._half_close_bidi_write_rpc(bidi_write_rpc)
           break
+    except StopIteration as e:
+      raise bidi_retry_util.BidiUploadStreamClosedError(
+          'The BiDi upload stream was unexpectedly closed.'
+      ) from e
     finally:
+      # Cancel the rpc if it is still open.
       bidi_write_rpc.close()
 
   def _get_request_for_creating_append_object(self):
@@ -288,7 +312,10 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
     )
 
     metadata_util.update_object_metadata_from_request_config(
-        destination_object, self._request_config, self._source_resource
+        destination_object,
+        self._request_config,
+        self._source_resource,
+        posix_to_set=self._posix_to_set,
     )
 
     write_object_spec = self._client.types.WriteObjectSpec(
@@ -374,6 +401,10 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
       self,
   ) -> resource_reference.ObjectResource:
     """Returns the object metadata after the upload."""
+    # For finalized objects, the object metadata is returned in the response of
+    # the last request(i.e. the request with finish_write set).
+    if self._finalized_resource:
+      return self._finalized_resource
     read_request = self._client.types.BidiReadObjectRequest(
         read_object_spec=self._client.types.BidiReadObjectSpec(
             bucket=self._initial_request.append_object_spec.bucket,
@@ -381,18 +412,10 @@ class _Upload(six.with_metaclass(abc.ABCMeta, object)):
             generation=self._initial_request.append_object_spec.generation,
         )
     )
-    bidi_rpc = gapic_util.MakeBidiRpc(
-        self._client,
-        self._client.storage.bidi_read_object,
-        initial_request=read_request,
-        metadata=metadata_util.get_bucket_name_routing_header(
-            grpc_util.get_full_bucket_name(
-                self._destination_resource.storage_url.bucket_name
-            )
-        ),
+    bidi_rpc = self._redirection_handler.start_bidi_rpc_with_retry_on_redirected_token_error(
+        read_request
     )
     try:
-      bidi_rpc.open()
       response = bidi_rpc.recv()
     finally:
       bidi_rpc.close()
@@ -417,18 +440,8 @@ class SimpleUpload(_Upload):
       CloudApiError: API returned an error.
     """
     self._initial_request = self._get_request_for_creating_append_object()
-    bidi_write_rpc = gapic_util.MakeBidiRpc(
-        self._client,
-        self._client.storage.bidi_write_object,
-        initial_request=self._initial_request,
-        metadata=metadata_util.get_bucket_name_routing_header(
-            grpc_util.get_full_bucket_name(
-                self._destination_resource.storage_url.bucket_name
-            )
-        ),
-    )
 
-    self._bidi_write_appendable_object(bidi_write_rpc)
+    self._bidi_write_appendable_object()
 
     return self._get_object_metadata_after_upload()
 
@@ -445,12 +458,6 @@ class ResumableUpload(_Upload):
     Raises:
       CloudApiError: API returned an error.
     """
-    metadata = metadata_util.get_bucket_name_routing_header(
-        grpc_util.get_full_bucket_name(
-            self._destination_resource.storage_url.bucket_name
-        )
-    )
-
     destination_object = self._get_object_if_exists()
     if destination_object:
       self._initial_request = (
@@ -458,12 +465,6 @@ class ResumableUpload(_Upload):
               destination_object
           )
       )
-      bidi_write_rpc = gapic_util.MakeBidiRpc(
-          self._client,
-          self._client.storage.bidi_write_object,
-          initial_request=self._initial_request,
-          metadata=metadata,
-      )
       self._start_offset = destination_object.size
       log.info(
           'Attempting to resume upload for object: %s at offset: %s',
@@ -471,7 +472,7 @@ class ResumableUpload(_Upload):
           destination_object.size,
       )
       try:
-        self._bidi_write_appendable_object(bidi_write_rpc, resuming_upload=True)
+        self._bidi_write_appendable_object(resuming_upload=True)
         return self._get_object_metadata_after_upload()
       except command_errors.HashMismatchError as e:
         log.info(e)
@@ -484,4 +485,5 @@ class ResumableUpload(_Upload):
         source_resource=self._source_resource,
         start_offset=0,
         delegator=self._delegator,
+        posix_to_set=self._posix_to_set,
     ).run()
