@@ -27,7 +27,9 @@ from googlecloudsdk.core.resource import resource_projection_spec
 from googlecloudsdk.core.resource import resource_projector
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA)
+@base.ReleaseTracks(
+    base.ReleaseTrack.GA, base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA
+)
 @base.DefaultUniverseOnly
 class List(base.ListCommand):
   """List Compute Engine network peerings."""
@@ -44,6 +46,7 @@ class List(base.ListCommand):
             peerMtu,
             importCustomRoutes,
             exportCustomRoutes,
+            updateStrategy,
             state,
             stateDetails
         )")
@@ -82,30 +85,4 @@ class List(base.ListCommand):
           peering['source_network'] = network.selfLink
         yield synthesized_network
 
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
-class ListAlpha(List):
-  """List Compute Engine network peerings."""
-
-  @staticmethod
-  def Args(parser):
-    parser.display_info.AddFormat("""
-        table(peerings:format="table(
-            name,
-            source_network.basename():label=NETWORK,
-            network.map().scope(projects).segment(0):label=PEER_PROJECT,
-            network.basename():label=PEER_NETWORK,
-            stackType,
-            peerMtu,
-            importCustomRoutes,
-            exportCustomRoutes,
-            updateStrategy,
-            state,
-            stateDetails
-        )")
-    """)
-
-    parser.add_argument(
-        '--network', help='Only show peerings of a specific network.')
-
 List.detailed_help = base_classes.GetGlobalListerHelp('peerings')
